/**************************************/
/*              Checksum              */
/* calculate byte checksum over file  */
/*------------------------------------*/
/*     (c) 1991 by Lukas Zeller       */
/**************************************/


/* Edition History
   ---------------
   001 : 92-03-18 : (full moon) created
*/

#include <stdio.h>
#include <ctype.h>
#include <modes.h>

extern int errno;

#define BSIZ 4096
char buffer[BSIZ];

/* global variables */
/* ---------------- */

int nargc;  /* number of non-option (and non argv[0]) arguments  */

int start=0;
int num=0;

/* main program */
main (argc,argv)

int argc;
char *argv[];

{
   int k, bytes;
   char **argp, *srcfile;

   long totbytes;
   int srcpath;
   unsigned short int chksum;

   /* --- entry point */
   options(argc,argv);
   if (nargc<1) {
      usage(argv[0]);
      exit (_errmsg(1,"at least one argument required\n"));
   }
   argp=&argv[1];

   while (nargc-->0) {
      while (*(srcfile=*(argp++))=='-'); /* get next non-option argument */

      if ((srcpath=open(srcfile,S_IREAD))<0)
         exit(_errmsg(errno,"Can't open file for read '%s'. ",srcfile));

      totbytes=0;
      chksum=0;
      lseek(srcpath,start,0);
      while((_gs_eof(srcpath)==0) && ((totbytes<num) || (num==0))) {
         if ((bytes=read(srcpath,buffer,BSIZ))<0)
            exit(_errmsg(errno,"Error reading file '%s'. ",srcfile));
         for (k=0; k<bytes; k++) {
            chksum+=(unsigned char) buffer[k];
            totbytes++;
            if ((totbytes>=num) && (num!=0)) break;
         }
      }
      close(srcpath);
      if ((totbytes<num) && (num!=0)) chksum+=(unsigned short int) ((num-totbytes)*0xFF);
      printf("File '%s': bytes=%ld, checksum=$%04X (%d)\n",
             srcfile,totbytes,chksum,chksum);
   }
} /* main */


/**** display program usage */
usage (pname)
char *pname;
{
  printf("Syntax:   %s <file> [<file>...]\n",pname);
  printf("Function: checksum calculation\n");
  printf("Options:  -?        : displays this text\n");
  printf("          -n=<n>    : calculate over <n> bytes, (empty=$FF)\n");
  printf("          -s=<n>    : start calculation at offset <n>\n");
  printf("(c) 1992 by L.Zeller\n");
} /* usage */


/**** analyze command line options */
options (argc,argv)
int argc;
char *argv[];
{
  int count, nextarg;
  char *sc;

  for (count=nargc=0;++count<argc;) {
    if (*(sc=argv[count])=='-') {
      /* this argument is an option (begins with a hyphen) */
      nextarg=0;
      while ((*(++sc)!=NULL)&&!nextarg) {
        switch (tolower(*sc)) {
          case '?' : usage(argv[0]); exit(0);
          case 's' : if (*(++sc)!='=') exit(_errmsg(1,"missing '=' for s option\n"));
                     start=atoi(++sc);
                     nextarg=1;
                     break;
          case 'n' : if (*(++sc)!='=') exit(_errmsg(1,"missing '=' for n option\n"));
                     num=atoi(++sc);
                     nextarg=1;
                     break;
          default  : usage(argv[0]); exit(_errmsg(1,"unknown option '%c'\n",*sc));
          /* options with arguments may set nextarg to stop processing
             of this argument. Otherwise, *sc must be the last char processed */
        } /* switch */
      } /* while */
    } /* if hyphen */
    else nargc++;
  } /* for */
} /* options */

