/*** list.c - for linked lists ***/

#include <stdio.h>
#include <math.h>
#include "header.h"

pfileptr make_pfile(filename)
char *filename;
{
   char *malloc();
   char *strcpy();
   pfileptr newptr;

   if ((newptr = (pfileptr)malloc(sizeof(pfile)))!=NULL) {
      strcpy(newptr->plotfile,filename);
      newptr->format = CONV;
      newptr->next = NULL;
      newptr->prev = NULL;
   }
   return(newptr);
}

/* Insert a plotfile at the tail of the current list */
pfileptr insert_pfile(filename)
char *filename;
{
   pfileptr prev,next,L,T;
   pfileptr make_pfile();

   L = pfiletail;
   if ((T=make_pfile(filename))!=NULL) {
      if (L==NULL) {
         pfilehead = T;
         pfiletail = T;
      } else {
         prev = L->prev;
         next = L->next;
         T->next = next;
         T->prev = L;
         L->next = T;
         if (next != NULL) next->prev = T;
         if (T->next == NULL) pfiletail = T;
      }
   }
   return(T);
}

/* print out the filenames */
void print_pfiles()
{
   extern pfileptr pfilehead;
   pfileptr L;
   for (L=pfilehead; L!=NULL; L=L->next)
      printf("%s\n",L->plotfile);
   printf("\n");
}

nodeptr make_node(x,y,z)
double x,y,z;
{
   char *malloc();
   nodeptr newptr;

   if ((newptr = (nodeptr)malloc(sizeof(node)))!=NULL) {
      newptr->x = x;
      newptr->y = y;
      newptr->z = z;
      newptr->next = NULL;
      newptr->prev = NULL;
   }
   return(newptr);
}

/* Insert a node at the tail of the current segment */
nodeptr insert_node(x,y,z,S)
double x,y,z;
segmptr S;
{
   nodeptr prev,next,L,T;
   nodeptr make_node();

   L = S->tail;
   if ((T=make_node(x,y,z))!=NULL) {
      if (L==NULL) {
         S->head = T;
         S->tail = T;
      } else {
         prev = L->prev;
         next = L->next;
         T->next = next;
         T->prev = L;
         L->next = T;
         if (next != NULL) next->prev = T;
         if (T->next == NULL) S->tail = T;
      }
   }
   return(T);
}

/* Delete node on segment S at address L */
void delete_node(L,S)
nodeptr L;
segmptr S;
{
   nodeptr prev,next;

   prev = L->prev;
   next = L->next;
   if (prev!=NULL) prev->next = next;
   if (next!=NULL) next->prev = prev;
   if (L==S->head) S->head = next;
   if (L==S->tail) S->tail = prev;
   free ((char*)L);
   L = NULL;
}

void print_nodes(L)
nodeptr L;
{
   for ( ; L!=NULL; L=L->next)
      printf("%10.5f %10.5f %10.5f\n",L->x,L->y,L->z);
   printf("\n");
}

void print_nodes_backward(L)
nodeptr L;
{
   for ( ; L!=NULL; L=L->prev)
      printf("%10.5f %10.5f %10.5f\n",L->x,L->y,L->z);
   printf("\n");
}

segmptr make_segm()
{
   char *malloc();
   segmptr newptr;

   if ((newptr = (segmptr)malloc(sizeof(segm)))!=NULL) {
      newptr->head = NULL;
      newptr->tail = NULL;
      newptr->next = NULL;
      newptr->prev = NULL;
      newptr->zave = 0.0;
   }
   return(newptr);
}

/* Insert a node at the tail of the current segment */
segmptr insert_segm()
{
   extern segmptr segmhead, segmtail;
   segmptr prev,next,L,T;
   segmptr make_segm();

   L = segmtail;
   if ((T=make_segm())!=NULL) {
      if (L==NULL) {
         segmhead = T;
         segmtail = T;
      } else {
         prev = L->prev;
         next = L->next;
         T->next = next;
         T->prev = L;
         L->next = T;
         if (next != NULL) next->prev = T;
         if (T->next == NULL) segmtail = T;
      }
   }
   return(T);
}

void delete_segm(S)
segmptr S;
{
   segmptr prev,next;

   prev = S->prev;
   next = S->next;
   if (prev!=NULL) prev->next = next;
   if (next!=NULL) next->prev = prev;
   if (S==segmhead) segmhead = next;
   if (S==segmtail) segmtail = prev;
   free ((char*)S);
   S = NULL;
}

/* print the segments */
void print_segms()
{
   extern  segmptr segmhead;
   segmptr S;
   void  print_nodes();

   S = segmhead;
   for ( ; S!=NULL; S=S->next)
      print_nodes(S->head);
   printf("\n");
}
