.TH PDRAW L "4 September 1990"
.SH NAME
pdraw - 3D plot program for X-windows and Postscript
.SH SYNOPSIS
.B
pdraw
[-v \fIvx vy vz\fR]
[-o \fIoptions-file\fR]
[-P\fIprinter\fR]
[-s \fIscale\fR] 
[-e]
[-h]
[-nosort]
[-ps]
plotfile1 plotfile2...
.SH DESCRIPTIONS
.PP
\fBPdraw\fR is a program for drawing 3D plots on X10 or X11 windows.
The program will also produce a POSTSCRIPT plot which
can be dumped out to an APPLE Laserwriter. 
\fBPdraw\fR reads in x-y-z data from a \fIplot-file\fR and
manipulates that data according to options specified
either in the command-line or in a \fIoptions-file\fR.
The \fIplot-file\fR can be compressed (see \fBcompress[1]\fR);
compressed files will be uncompressed automatically.
The program then plots lines on a screen or dumps the plots 
to a POSTSCRIPT file.  
.PP
The \fIplot-file\fR input data
consists of alternating x, y and z values, in the format shown below.
.RS
.LP
.B
Data File Format (plot-file)
.LP
.nf
 xmin xmax ymin ymax zmin zmax
 ncurves
 npts 
 z1
 z2
 ..
 ..
 ..
 npts 
 z1
 z2
 ..
 ..
 ..
.fi
.RE
.LP
In the above, \fIxmin\fR, \fIxmax\fR, \fIymin\fR, \fIymax\fR, \fIzmin\fR
and \fIzmax\fR are
lower and upper bounds of the desired plot, \fIncurves\fR are the number of
curves to be plotted, and \fInpts\fR are the number of points in each
curve.  The data file can consist of more than one set of curves to be
plotted; each set (i.e. one set for each separate graph) is separated from
the next by a blank line.
.LP
Upon starting up the program, \fBpdraw\fR will read in the data stored in
the \fIplot-file\fR, as well as any plotting options specified either in
the command line or in the \fIoptions-file\fR.
\fBPdraw\fR then uses the given view direction to rotate and transform the
plot onto a plane perpendicular to the viewing vector.  Currently, only
parallel projection is supported.  If the program is being run under
X-windows, the plot will then be drawn on the screen.
The viewing vector can be changed using the "H", "J", "K", "L" and "O" keys
on the the keyboards; the plot on the screen can be rotated
sideways using the "H" or "L" keys, rotated up or down using 
the "J" and "K" keys, and drawn with the original viewing vector using "O".
The "A", "S", "D" and "F" keys will produce 90\(de rotations.
"Z" will plot the image projected on the x-y plane (z=constant),
"Y" will plot the image projected on the x-z plane (y=constant),
and "X" will plot the image projected on the y-z plane (x=constant).
The final view angle will be saved and used for the POSTCRIPT plot.
Finally, the user will be
prompted as to whether or not the POSTSCRIPT plot is to be sent to a printer.
.SH OPTIONS
.TP 5
.B \-v \fIvx vy vz\fR
reads in the viewing eye position, relative to (0,0,0).
The plot will be rotated and transformed so that the z-axis is parallel to
this position.  For example, a view position of (1,0,0) means that the 3D
structure is being viewed with parallel projection from the x-axis.
.TP 5
.B \-o \fIoptions-file\fR
reads plotting options from the file \fIoptions-file\fR.
Each option specification consists of a keyword and its corresponding value.
The parser recognizes only a limited set of keywords; their values are
either numbers, quoted strings, or the words "on" and "off".
All the words in the option specification must be on the same line.
The pound sign (#) indicates that the remainder of the line is a comment to
be ignored by the parser.
.RS
.LP
.B
List of Options (options-file)
.RS
.LP
xlabel "LABEL" 	#[default = "X-Axis"] 		- for the x-label   
.br
ylabel "LABEL" 	#[default = "Y-Axis"] 		- for the y-label
.br
zlabel "LABEL"		#[default = "Z-Axis"] 		- for the z-label
.br
toplabel "LABEL" 	#[default = "3D Line Plot"] 	- for the top-label  
.br
equalscale on/off 	#[default = on] 			- for equal x-y scaling 
.br
postscript on/off 		#[default = on] 			- for postscript (PS) plot 
.br
printplot on/off 		#[default = off] 			- send PS file to printer 
.br
noplot on/off 		#[default = off] 			- no graphics plot 
.br
printer "PRINTER"	#[default = $PRINTER]		- define the printer
.br
line on/off 		#[default = on] 			- draw the line
.br
linechange on/off 	#[default = off] 			- change the linetypes
.br
marker on/off 		#[default = off] 			- draw the marker
.br
markerchange on/off 	#[default = off] 			- change the markertypes
.br
hiddenline on/off 	#[default = off] 			- for hidden-line drawings
.br
nosort on/off 		#[default = off] 			- for hidden-line drawings
.br
scale [0.1 - 1.0] 		#[default = 1.00] 		- scales the PS plot  
.br
xticks [1 - 20] 		#[default = 2] 			- no. of x-divisions  
.br
yticks [1 - 20] 		#[default = 2] 			- no. of y-divisions   
.br
zticks [1 - 20] 		#[default = 2] 			- no. of z-divisions   
.RE
.RE
.TP 5
.B \-P\fIprinter\fR
specifies which printer to which to send the postscript plot.
The current default sets the printer name to the environment variable
$PRINTER.
If this variable is not set, then the printer used is
the lp550M printer in 550M Cory.
.TP 5
.B \-s \fIscale\fR
sets a scale factor.  This is used only for POSTSCRIPT plotting.
.TP 5
.B \-e 
forces \fIunequal\fR scales to be applied to the x, y and z axes. 
The boundary of the 3D object will then resemble a cube.
.TP 5
.B \-h 
draws polygons with hidden-lines.
.TP 5
.B \-nosort
prevents sorting for the hidden-line option.
.TP 5
.B \-noplot
prevents plots on the graphics display.  
.TP 5
.B \-print
sends plots to the printer automatically.
.TP 5
.B \-ps
turns the postscript plotting option off.
.TP 5
.I host:display
opens a window on the given host and display
.TP 5
.B \=geom 
.TP 5
.B \-rv 
.TP 5
.B \-bw \fIborder-width\fR
.TP 5
.B \-bd \fIcolor\fR 
.TP 5
.B \-fg \fIcolor\fR 
.TP 5
.B \-bg \fIcolor\fR
sets input options for the X-window system.
.SH BUGS  
Not really that many.  The POSTSCRIPT labels need to be adjusted.
The labels don't come out well when the picture is rotated beyond the default 
view.  The parser needs to be improved.  There should be a better way to
put change linetypes and markertypes.  Log axes might be nice.
Also should incorporate drawplot modifications here.
.SH AUTHOR
Kenny K.H. Toh (ktoh@mascot.berkeley.edu)
.SH FILES
\fIdataplot.ps\fR 	temporary POSTSCRIPT file
.SH SEE ALSO
contour(L), drawplot(L)
