/***   sort.c - for sorting   ***/

#include <stdio.h>
#include "header.h"
/*****************************************************
*          Implementation of Quicksort algorithm     *
*          By Edward W. Scheckler   August 3, 1990   *
*	            EECS Dept.  UC-Berkeley	     *
*	   					     *
*	   For complicated plots, this algorithm     *
* 	   runs in approx. 2NlnN time, which is      *
*	   orders of magnitude faster then bubble    *
*	   sort methods running as N**2              *
*	   This algorithm is not very efficient for  *
*	   lists that are already mostly in order.   *
*	   In those cases, bubble sort is best       *
*	   so do_quick_sort first checks to see      *
*	   if the data is almost ordered.            *
*****************************************************/
do_quick_sort()
{
   xyzdata transform_point();
   nodeptr insert_node();

   extern int nosort;
   extern segmptr segmhead;

   int ii,npts, sorted;
   int narray,in,sort_times;
   xyzdata point,newpt;
   double  ztot, dx, dy, dz;
   segmptr S,A,B,C;
   segmptr *a;
   segmptr *allocate_quicksort_array();
   nodeptr Nd, Nd1, Nd2;
   /* put in a new point to make sure that the polygon is closed */
   narray = 0;
   for (S=segmhead; S!=NULL; S=S->next) {
      narray++;
      if (((Nd1=S->head)!=NULL) && ((Nd2=S->tail)!=NULL)) {
         dx = Nd1->x - Nd2->x;
         dy = Nd1->y - Nd2->y;
         dz = Nd1->z - Nd2->z;
         if ((dx*dx + dy*dy + dz*dz) > SMALL) {
            insert_node(Nd1->x,Nd1->y,Nd1->z,S);
         }
      }
   }
   if (nosort) return;
   /*allocate an array for holding pointers to the segm's */
   a = allocate_quicksort_array(narray);

   /* first calculate the z-ave value of all the points in the list */
   /* also set the elements of the segmptr array a                  */
   in = 0;
   for (S=segmhead; S!=NULL; S=S->next) {
      *(a + in) = S;
      in ++;
      ztot = 0.0;
      npts = 0;
      for (Nd=S->head; Nd->next!=NULL; Nd=Nd->next) {
         /* count only n-1 points, last point is assumed same as 1st point */
         point.x = Nd->x;
         point.y = Nd->y;
         point.z = Nd->z;
         newpt = transform_point(point,view_transfo);
         ztot = ztot + newpt.z;
         npts++;
      }
      if (npts>0) S->zave = ztot/(double)npts;
   }
   /*first check if already sorted*/
   sort_times = 0;
   do {
      sorted = TRUE;
      for (S=segmhead; S->next!=NULL; S=S->next) {
         /* compare each adjacent pair of elements */
         B = S->next;
         if (S->zave > B->zave) {
            A = S->prev;
            C = B->next;
            if (S==segmhead) segmhead = B;
            if (B==segmtail) segmtail = S;
            if (A!=NULL) A->next = B;
            B->next = S;
            B->prev = A;
            S->next = C;
            S->prev = B;
            if (C!=NULL) C->prev = S;
            S = S->prev;
            sorted = FALSE;
         }
      }
      sort_times++;
   } while (!sorted && sort_times < 5);
   if(sorted){
   free((char *)a);
   return(0);
   }

   /*do the quicksort*/
   quicksort(a,0,narray-1,narray-1);
   /*put the sorted array elements back into the segmenet list*/
   
   /*
   in = 0;
   for (S=segmhead; S!=NULL; S=S->next) {
	S->head = a[in]->head;
	S->tail = a[in]->tail;
	S->zave = a[in]->zave;
	if(in+1 < narray){
		S->next = a[in+1];
	} else {
		S->next = NULL;
	}
	if(in-1 >= 0){
		S->prev = a[in-1];
	} else {
		S->prev = NULL;
	}
	in++;
   }
   */
   segmhead = a[0];
   segmtail = a[narray-1];
   for(in = 0; in < narray; in++){
	if(in+1 < narray){
	a[in]->next = a[in+1];
	} else {
	a[in]->next = NULL;
	}
	if(in-1 >= 0 ){
	a[in]->prev = a[in-1];
	}else {
	a[in]->prev= NULL;
	}
   }

   free((char *)a);
   
}
segmptr *allocate_quicksort_array(narr)
int narr;
{
	char *malloc();
	segmptr *newptr;
	unsigned int arr_size;
	arr_size = (unsigned)(narr)*sizeof(segm);
	newptr = (segmptr *)malloc(arr_size);
	return(newptr);
}
quicksort(a,l,r,narr)
segmptr *a;
int l,r,narr;
{
	int i,j;
	int p,q;
	int sortflag;
	segmptr v,t,ta;
	/*if the list is small, do an insertion sort for speed*/
	if(r-l > 5){
		v = a[r];
		i = l - 1;
		j = r;
		do{
			do{
				i++;
			} while(i< narr &&!(a[i]->zave >= v->zave));
			do{
				j--;
			}while( j > 0 && !(a[j]->zave <= v->zave));
			t = a[i];
			a[i] = a[j];
			a[j] = t;
		}while(!(j <= i));
			a[j] = a[i];
			a[i] = a[r];
			a[r] = t;
			quicksort(a,l,i-1,narr);
			quicksort(a,i+1,r,narr);
	} else {
		do{
		   sortflag = 0;
		   for(p=l;p<r;p++){
		      if(a[p]->zave > a[p+1]->zave){
			  ta = a[p];
			  a[p] = a[p+1];
			  a[p+1] = ta;
			  sortflag = 1;
		      }
		   }
	        }while(sortflag != 0);
	}

}	
/* sort_points() does an insertion sort */
bubble_sort() 
{
   xyzdata transform_point();
   nodeptr insert_node();

   extern  int     nosort;
   extern  segmptr segmhead;

   int     npts, sorted;
   xyzdata point,newpt;
   double  ztot, dx, dy, dz;
   segmptr S,A,B,C;
   nodeptr Nd, Nd1, Nd2;

   /* put in a new point to make sure that the polygon is closed */
   for (S=segmhead; S!=NULL; S=S->next) {
      if (((Nd1=S->head)!=NULL) && ((Nd2=S->tail)!=NULL)) {
         dx = Nd1->x - Nd2->x;
         dy = Nd1->y - Nd2->y;
         dz = Nd1->z - Nd2->z;
         if ((dx*dx + dy*dy + dz*dz) > SMALL) {
            insert_node(Nd1->x,Nd1->y,Nd1->z,S);
         }
      }
   }
   if (nosort) return;

   /* first calculate the z-ave value of all the points in the list */
   for (S=segmhead; S!=NULL; S=S->next) {
      ztot = 0.0;
      npts = 0;
      for (Nd=S->head; Nd->next!=NULL; Nd=Nd->next) {
         /* count only n-1 points, last point is assumed same as 1st point */
         point.x = Nd->x;
         point.y = Nd->y;
         point.z = Nd->z;
         newpt = transform_point(point,view_transfo);
         ztot = ztot + newpt.z;
         npts++;
      }
      if (npts>0) S->zave = ztot/(double)npts;
   }

   /* now sort according to average z value */
   /* the first structure should have the smallest z-value */
   do {
      sorted = TRUE;
      for (S=segmhead; S->next!=NULL; S=S->next) {
         /* compare each adjacent pair of elements */
         B = S->next;
         if (S->zave > B->zave) {
            /* A -> S -> B -> C */
            /* A -> B -> S -> C */
            A = S->prev;
            C = B->next;
            if (S==segmhead) segmhead = B;
            if (B==segmtail) segmtail = S;
            if (A!=NULL) A->next = B;
            B->next = S;
            B->prev = A;
            S->next = C;
            S->prev = B;
            if (C!=NULL) C->prev = S;
            S = S->prev;
            sorted = FALSE;
         }
      }
   } while (!sorted);
}

